# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from elisa.core.utils import misc

class HistoryItem(object):
    """The basic tree element.
    """

    _index = -1

    def _get_active_index(self):
        #print '>>', self._index, self
        return self._index

    def _set_active_index(self, new_index):
        #print '<<', new_index, self
        self._index = new_index

    _active_index = property(fget=_get_active_index,
                             fset=_set_active_index)


    def __init__(self, menuitem):
        self._parent = None          # The parent Item
        self._children = []          # The child Item(s)
        self._nb_children = 0        # Number of child Item(s)
        #self._active_index = 0      # The active child index
        self._widget = None        # The gui widget representing the Item
        self._menuitem = menuitem    # The MenuItem, None if not in the path
        self._parsed = False         # The Item has already been parsed
        self._level_widget = None

    def find(self, menu_item):
        if self.get_menuitem() == menu_item:
            return self
        else:
            for child in self.get_children():
                found = child.find(menu_item)
                if found:
                    return found
        return None


    def pprint(self, d=0):
        if not d:
            print repr(self)
        d += 1
        for child in self.get_children():
            spc = "  " * d
            print "%s- %r" % (spc, child)
            child.pprint(d)
        
    def __repr__(self):
        name = self.get_name()
        parent_name = ''
        parent = self.get_parent()
        if parent:
            parent_name = parent.get_name()
        if parent_name not in ('unknown',''):
            parent_name = "parent=%r" % parent_name
        return "<Item instance at %s: %r %s, %s children>" % (hex(-id(self)),
                                                              name, parent_name,
                                                              self._nb_children)

    def get_name(self):
        name = "unknown"
        if self.get_menuitem() and self.get_menuitem().get_short_name():
            name = misc.fs_encode(self.get_menuitem().get_short_name())
        return name

    def get_parent(self):
        return self._parent

    def set_parent(self, parent):
        self._parent = parent

    def get_widgets(self):
        widgets = []
        for child in self.get_children():
            widget = child.get_widget()
            if not widget:
                widget = child.get_level_widget()
            if widget:
                widgets.append(widget)
            child_widgets = child.get_widgets()
            if child_widgets:
                widgets.extend(child_widgets)
        return widgets

    def get_children(self):
        return tuple(self._children)

    def add_child(self, item, index=None):
        if not item.get_parent():
            item.set_parent(self)
        if item in self._children:
            return
        if index == None:
            self._children.append(item)
        else:
            self._children.insert(index, item)
        self._nb_children += 1

    def del_child(self, item):
        if self._nb_children != 0:
            self._children.remove(item)
            self._nb_children -= 1
    
    def get_nb_children(self):
        return self._nb_children

    def get_active_index(self):
        r = self._active_index
        return r

    def set_active_index(self, index):
        self._active_index = index

    def get_active_child(self):
        index = self.get_active_index()
        if index > -1:
            child = self._children[index]
        elif len(self._children):
            child = self._children[0]
        else:
            child = self
        return child

    def set_previous_child_active(self):
        new_index = self._active_index - 1
        if new_index < 0:
            # go to end of list
            # new_index = self.get_nb_children() -1
            child = None
        else:
            self._active_index = new_index
            child = self._children[new_index]
        return child

    def set_next_child_active(self):
        new_index = self._active_index + 1
        if new_index >= self._nb_children:
            # go back to beginning of list
            #new_index = 0
            child = None
        else:
            self._active_index = new_index
            child = self._children[new_index]
        return child

    def get_widget(self):
        return self._widget

    def set_widget(self, widget):
        self._widget = widget


    def get_level_widget(self):
        return self._level_widget

    def set_level_widget(self, widget):
        self._level_widget = widget

    def has_level(self):
        return self.get_level_widget() is not None

    def get_menuitem(self):
        return self._menuitem

    def set_menuitem(self, menuitem):
        self._menuitem = menuitem

    def get_parsed(self):
        return self._parsed

    def set_parsed(self, parsed):
        self._parsed = parsed
