# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

"""
@newfield emits: Corpus, Corpora
"""

from elisa.core.plugin import IPlugin

class IDataAccess(IPlugin):

    def create_child_menuitems_for_uri(uri, start_item, nb_items, for_plugin):
        """
        return a list of menuitem for each child element of name (uri)

        :param uri: the URI to scan
        :type uri: string
        :param start_item: the index of the first item
        :type start_item: int
        :param nb_items: the number of items to load (-1 == all)
        :type nb_items: int
        :param for_plugin: the plugin for which the menu items are created
        :type for_plugin: plugin.Plugin
        :rtype: list:menu.MenuItem
        """

    def create_menu_for_uri(uri, for_plugin, short_name=None):
        """

        :param uri: the URI to scan
        :type uri: media_uri.Uri
        :param for_plugin: the plugin for which the menu item is created
        :type for_plugin: plugin.Plugin
        :keyword short_name: the label of the menu item
        :type short_name: string or None
        :rtype: menu.MenuItem or None
        """

    def validate_uri(uri):
        """
        Return True if the uri is well-formed and valid against the plugin.

        When an URI is valid, the plugin will be able to process it correctly.

        :param uri: the URI to validate
        :type uri: media_uri.Uri
        :rtype: bool
        """

    def is_dir(uri):
        """
        return True if a directory
        
        :param uri: the URI to analyze
        :type uri: media_uri.Uri
        :rtype: bool
        """

    def has_children(uri):
        """
        Detect whether the given uri has children

        :param uri: the URI to scan
        :type uri: media_uri.Uri
        :rtype: bool
        """

    def get_parent(uri):
        """
        Return the parent uri of the given uri

        :param uri: the URI to scan
        :type uri: media_uri.Uri
        :rtype: string
        """
        
    def get_direct_children(uri):
        """
        Scan the data located at given uri and return the direct
        children if the uri represents a directory
        
        :param uri: the URI to analyze
        :type uri: media_uri.Uri
        :rtype: list of (child_type, file_type, fname, index)
        :type child_type: string ('directory' or 'file')
        :type file_type: string ('' or 'audio' or 'video')
        :type fname: string
        :type index: int
        """

    def get_file_type(uri):
        """
        Return the type of file. If it's a directory return empty
        string, either return the content-type of the uri

        :param uri: the URI to analyze
        :type uri: media_uri.Uri
        :rtype: string
        """
        
    def open(uri, mode=None):
        """
        Open an uri and return DataAccessFile file

        :param uri: the URI to open
        :type uri: media_uri.Uri
        :keyword mode: how to open the file -- see manual of builtin open()
        :type mode: string or None
        :rtype: DataAccessFile        
        """

    def close(data_access_file):
        """
        Close a DataAccessFile

        :param data_access_file: the file to close
        :type data_access_file: DataAccessFile
        """

    def seek(data_access_file, offset, whence=0):
        """
        Seek data in a DataAccessFile

        :param data_access_file: the file to seek in
        :type data_access_file: DataAccessFile
        :param offset: how many bytes to seek
        :type offset: int
        :keyword whence: from where to seek
        :type whence: int (default=0)
        """

    def read(data_access_file, size=-1):
        """
        Read data from a DataAccessFile
        
        :param data_access_file: the file to read data from
        :type data_access_file: DataAccessFile
        :keyword size: how many data we should try to read
        :type size: int (default: -1 == all)
        :rtype: string
        """

    def remove_location(uri):
        """
        Delete data at location identified by given uri

        :param uri: the URI representing the file or directory to delete
        :type uri: media_uri.Uri
        """

    def next_location(uri, root=None, at_user_request=False):
        """
        Return the uri just next to `uri` and record it to history

        :param uri: the URI representing the file or directory from where
                    to move on
        :type uri: media_uri.Uri
        :keyword root: root URI if not None, next_location will be
                       recorded to history
        :type root: media_uri.Uri
        :param at_user_request: flag indicating if the user requested the next location or if the player is just  acting on his own after an EOS
        :type at_user_request: bool
        :rtype: (location: string, short_name: string)
        """

    def previous_location(uri):
        """
        Return the uri recorded in history before `uri`

        :param uri: the URI representing the file or directory from where
                    to move on
        :type uri: media_uri.Uri
        :rtype: (location: string, short_name: string)
        """
        
    def monitor_uri(self, uri, callback):
        """
        Start monitoring `uri` for modification and call function `callback`
        in case of any change happening on `uri`

        :param uri: the URI representing the file or directory to monitor
        :type uri: media_uri.Uri
        :param callback: a callable taking the event that occured and the uri
                        of the file on which the event applies to
                        event can be one of "ADDED", "REMOVED", "CHANGED"
                        # FIXME: should be defined as constants elsewhere
        """

    def unmonitor_uri(self, uri):
        """
        Stop monitoring `uri`

        :param uri: the URI representing the file or directory to monitor
        :type uri: media_uri.Uri        
        """
 
    def uri_is_monitorable(self, uri):
        """
        Return if the `uri` is monitorable for modification
        
        :param uri: the URI representing the file or directory for which we would
                    like to know if it is monitorable or not
        :type uri: media_uri.Uri
        :rtype: bool
        """

    def uri_is_monitored(self, uri):
        """
        Return if the `uri` is currently monitored for modification
        
        :param uri: the URI representing the file or directory for which we would
                    like to know if it is currently monitored or not
        :type uri: media_uri.Uri
        :rtype: bool
        """


class IMediaPlayer(IPlugin):
    """
    XXX: this interface needs to be refactored.
    """

    def folder_action_menu(menu_item):
        """
        """

    def item_action_menu(menu_item):
        """
        """

    def play_item(menu_item):
        """
        """
        
    def get_player(menu_item):
        """
        """
        
    def pause_item(menu_item):
        """
        """

    def seekf_item(menu_item):
        """
        """

    def seekb_item(menu_item):
        """
        """

    def stop_item(menu_item):
        """
        """

    def stopped(menu_item):
        """
        """

    def playing(menu_item):
        """
        """

    def en_queue_item(menu_item):
        """
        """

    def play_all_item(menu_item):
        """
        """

class IQueue(IPlugin):

    def get_current_item():
        """
        Return the current item of the playing queue

        :rtype: (uri: media_uri.Uri, short_name: string)
        """

    def set_current_item(uri, short_name):
        """
        Set the current uri of the playing queue

        :param uri: the current uri
        :type uri: media_uri.Uri
        :param short_name: the short_name of the item
        :type short_name: string
        """

    def next_item(head_uri=None, current=None):
        """
        Select the next uri of the queue

        :keyword head_uri: the head URI of the queue
        :type head_uri: media_uri.Uri
        :keyword current: the current URI of the queue
        :type current: media_uri.Uri
        :rtype: (next_location: string, short_name: string)
        """

    def previous_item():
        """
        Select the previous uri of the queue

        :rtype: (previous_location: string, short_name: string)
        """


class IUserInputPlugin(IPlugin):

    def get_input_events():
        """
        Wait for one event in the input queue and create a new Message
        for the bus.
        """

    def create_pgm_render_event(event_type):
        """
        Create an event of the given type

        :keyword event_type: the type of event
        :type event_type: pgmrender.PgmRenderEventType
        :rtype: pgmrender.PgmRenderEvent
        """


class ISkin(IPlugin):
    """
    

    """

    def get_menu_manager():
        """
        Access to the Menu manager

        :return: the manager responsible for handling MenuItems and their widgets
        :rtype: MenuManager instance
        """

    def get_picture_path(relative_path):
        """
        Given a picture filename, return its full path from skin's
        pictures directory.

        :param relative_path: relative path of the picture. ex: 'foo.png'
        :type relative_path: string
        :return: the full path of the picture. ex: '/path/to/foo.png'
        :rtype: string
        """
        
    def is_icon(picture_path):
        """
        Does the given picture absolute path point to a skin icon?

        :param picture_path: the absolute path of the picture. ex: '/path/foo.png'
        :type picture_path: string
        :rtype: bool
        """

    def display_image(uri):
        """
        Display an image given its URI

        :param uri: the URI to display
        :type uri: string
        """

    def display_stream(player, uri):
        """
        Display a stream given its URI

        :param uri: the stream URI to display
        :type uri: string
        """

    def rotate_image(uri, angle):
        """
        Rotate picture located at given URI and update widgets accordingly

        :param uri: the URI to rotate
        :type uri: string
        :param angle: angle rotation, positive => clockwise.
        :type angle: float
        """

    def prepare():
        """
        Build the UI with widgets
        """
                
    def show_menu(self):
        """
        Show the menu with the current selection.
        
        :emits: core.signals.skin.showing_menu
        """

    def hide_menu():
        """
        Hide the menu tree.

        :emits: core.signals.skin.hiding_menu
        """

    def activate():
        """
        Activate the current selected menu_item

        :rtype: bool
        """

    def move_left():
        """
        Move the current selection one item on the left

        :rtype: bool
        """

    def move_right():
        """
        Move the current selection one item on the right

        :rtype: bool
        """

    def move_down():
        """
        Go one menu level down, if possible.

        :rtype: bool
        """

    def move_up():
        """
        Go to the upper level if it's not the root menu

        :rtype: bool
        """

    def toggle_fullscreen():
        """
        Toggle the fullscreen state of the window
        
        """

    def toggle_menu():
        """
        Toggle the visible state of the menu
        """

    def notify(message):
        """
        Display a message to the user
        """

    def add_widget(widget):
        """
        Add a new widget in the window
        """

    def remove_widget(widget):
        """
        Remove a widget from the window
        """

class IServerProvider(IPlugin):
    """
    Provides a netword server
    """

    def get_host_address():
        """
        Retrieve host address
        """

class IResourceProvider(IPlugin):
    """
    Provides a twisted resource.Resource object to be used
    by the HTTP server
    """

    def get_resource():
        """
        Retrieve web resource
        """
