# -*- coding: utf-8 -*-
# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php

# Copyright (C) 2006,2007, Elisa team <contact@fluendo.com>

from elisa.core.plugin import IPlugin
from elisa.core.interfaces import IResourceProvider
from elisa.core.base_plugins import TreePlugin
from elisa.core import menu, common
from elisa.core import widgets
from elisa.core.signals.application import application_started
import elisa.core.utils.misc
from elisa.core.widgets.list import SCROLL_NORMAL, SCROLL_MIDDLE
from elisa.core.widgets import textarea
from elisa.core.signals.menu import active_changed
from elisa.core.signals.misc import state_changed

from pgm.layout import group
from pgm import widget
from pgm.timing import controller, modifier, keyframe
from pgm.form import image

from twisted.web import resource, static, server
from twisted.python import util
import os

class ConfigResource(resource.Resource):

    def __init__(self):
        resource.Resource.__init__(self)
        #self.addSlash = True                        
        
    def render(self, request):
        return """
        <html>
        <head>
        <title>Elisa Configuration page</title>
        </head>
        <body>
        <h1>Configuration page!</h1>
        </body>
        </html>
        """

    def getChild(self, path, request):
        if (path == ''):
            return self
        else:
            return ConfigResource()
        
class ConfigTreePlugin(TreePlugin):
    """


    """

    __implements__ = IResourceProvider
    provides = ['http:config']

    name = "config"
    menu_name = "Config"
    menu_help_string = "Configure Elisa"

    def __init__(self):
        TreePlugin.__init__(self)
        self._config_updated = False
        self.set_action_callback(self.show_panel)
        application_started.connect(self._application_started)
        
    def _application_started(self):
        self._skin = common.get_application().get_skin()
        self.prepare_panel()

    def add_wizard_shortcuts(self):
        pass
        
    def get_resource(self):
        return ConfigResource()

    def touch(self):
        self._config_updated = True

    def restart_required(self):
        return self._config_updated
        

    def prepare_panel(self):
        if not self._skin:
            return
        self._panel = Panel(self._skin, self)

    def show_panel(self, menu_item):
        if not self._skin:
            return
        
        self._skin.hide_menu()
        self._skin.reset_background()
        self._skin.add_widget(self._panel)

        def cb(ctr):
            self._panel.connect()
            
        self._panel.animate(cb, True)

    def hide_panel(self):
        
        def cb(ctr):
            self._panel.disconnect()
            self._skin.remove_widget(self._panel)
            self._skin.show_menu()
            
            if self.restart_required():

                def on_cancel():
                    pass

                def on_ok():
                    common.get_application().restart()
                
                msg = _('The configuration has been updated. '\
                        'Do you want to restart Elisa now?')
                self._skin.open_popup(msg, on_ok, on_cancel)
                
        self._panel.animate(cb, False)
        
class Panel(group.Group):

    def __init__(self, skin, config_plugin):
        self._skin = skin
        self._config_plugin = config_plugin
        win = skin.get_window()
        group.Group.__init__(self, size=win.size, position=(0,0))

        self._text_area = None
        self._button_size = (268, 50)
        self._arrow_size = (250, 32)
        self._menu_mapping = {}        
        prev_path = self._skin.get_picture_path('../config/arrow_up_on.png')
        next_path = self._skin.get_picture_path('../config/arrow_down_on.png')
        left_bg_path = self._skin.get_picture_path('../config/left.png')
        right_bg_path = self._skin.get_picture_path('../config/right.png')
        self._button_path = self._skin.get_picture_path('../config/button_black.png')
        self._button_enabled_path = self._skin.get_picture_path('../config/button_select.png')
        self._button_img = image.Image(self._button_path)
        list_size = (self._button_size[0], 400)
        self._button_list = widgets.list.VList(size=list_size,
                                               spacing=20,
                                               background_picture_path=left_bg_path)

        # prev button
        prev_menu_item = menu.MenuItem(short_name="")
        prev_menu_item.set_icon_path(prev_path)
        prev_button = self.create_button(prev_menu_item)

        # next button
        next_menu_item = menu.MenuItem(short_name="")
        next_menu_item.set_icon_path(next_path)
        next_button = self.create_button(next_menu_item)

        self._root_menu = menu.MenuTree(short_name="config_menu")

        # save button
        save_item = menu.MenuItem(short_name=_("Save"))
        save_item.set_action_callback(self._save)
        save_item.set_help_string("Save and exit from the configuration interface")
        self._root_menu.add_item(save_item)
        self.add_button(save_item)

##         # wizard
##         wizard_item = menu.MenuItem(short_name=_("Wizard"))
##         wizard_item.set_action_callback(self._start_wizard)
##         wizard_item.set_help_string("Wiizz")
##         self.add_button(wizard_item)

        # plugins
        plugins_item = menu.MenuItem(short_name=_("Plugins"))
        plugins_item.set_action_callback(self._show_plugins)
        plugins_item.set_help_string(_("Configure plugins"))
        self._root_menu.add_item(plugins_item)
        self.add_button(plugins_item)

        self.build_plugins_list(plugins_item)

        self._button_list.set_scrollbars(9, prev_button, next_button,
                                         SCROLL_NORMAL)
        self.add(self._button_list)
        
        self._text_area = textarea.TextArea((200,600),10, "foo",
                                            background_picture_path=right_bg_path,
                                            columns=20)
        self.add(self._text_area)
        
        self.display_help(save_item.get_help_string())
        state_changed.connect(self._state_changed)

    def _state_changed(self, obj, property_name, new_value):
        if obj not in self._menu_mapping.values():
            return
        if property_name == 'enabled':
            if new_value:
                obj.set_icon_path(self._button_enabled_path)
            else:
                obj.set_icon_path(self._button_path)
            
    def build_plugins_list(self, plugins_item):
        plugin_manager = common.get_plugin_manager()
        plugins = plugin_manager.get_plugins()
        for entry_point_name, plugins in plugins.iteritems():
            for plugin in plugins:
                plugin_item = menu.MenuItem(short_name=plugin.name)
                # FIXME: this is dummy
                try:
                    help_str = getattr(plugin, 'menu_help_string')
                except AttributeError:
                    help_str = ''
                plugin_item.set_help_string(help_str)
                plugin_item.set_action_callback(self._show_plugin)
                plugin_item.set_enabled(plugin.enabled)
                if plugin.enabled:
                    plugin_item.set_icon_path(self._button_enabled_path)
                self.build_plugin_menu(plugin_item, plugin)
                plugins_item.add_item(plugin_item)

    def build_plugin_menu(self, plugin_menu_item, plugin):
        #print plugin.get_config()
        toggle = menu.MenuItem(short_name='')
        self._update_toggle_plugin(None, toggle, plugin.enabled, plugin.name)
        toggle.set_action_callback(self._toggle_plugin)
        plugin_menu_item.insert_item(0, toggle)
        
    def active_changed(self, old_widget, new_widget, step):
        current_item = self._button_list.current_item
        menu_item = self._menu_mapping.get(current_item)
        if menu_item:
            self.display_help(menu_item.get_help_string())

    def display_help(self, message):
        if self._text_area:
            self._text_area.text = message

    def animate(self, cb, visible):
        """
        Slide the two sides of the panel horizontally
        """
        win_width = self._skin.get_window().width
        middle = 430
        left_margin = 20
        if visible:
            start_left = -win_width
            end_left = left_margin
            start_right = win_width
            end_right = middle
        else:
            start_left = left_margin
            end_left = -win_width
            start_right = middle
            end_right = win_width

        timeline_left = [ keyframe.KeyFrame(start_left, 0),
                          keyframe.KeyFrame(end_left, 1) ]

        timeline_right = [ keyframe.KeyFrame(start_right, 0),
                           keyframe.KeyFrame(end_right, 1) ]
        
        prop = 'x'
        duration = 2000
        mod_left = modifier.Modifier([self._button_list,],
                                     prop, timeline_left)

        mod_right = modifier.Modifier([self._text_area,], prop, timeline_right)
        
        self._ctrl = controller.Controller(begin=0, duration=duration,
                                           resolution=10,
                                           modifiers=[mod_left,
                                                      mod_right
                                                      ],
                                           end_behavior=controller.END,
                                           interpolation=controller.SINUSOIDAL,
                                           end_callback=cb)
        self._ctrl.start()
        
        
    def add_button(self, menu_item, index=-1, leaf=False):
        """
        Create the button corresponding to a menu_item and add it at givent
        index of the buttons list.
        """
        already_added = len([b for b in self._button_list
                             if isinstance(b, widgets.button.Button) and \
                             b.text == menu_item.get_short_name()]) > 0
        button = self.create_button(menu_item)
        if not already_added:
            self._button_list.add(button, index)
                
        self._menu_mapping[button] = menu_item
        
    def create_button(self, menu_item):
        """
        Create a button corresponding to a menu_item
        """
        icon_path = menu_item.get_icon_path()
        if not icon_path:
            icon_path = self._button_path
            
        icon = image.Image(icon_path)
        button = widgets.button.Button(size=self._button_size,
                                       text=menu_item.get_short_name(),
                                       icon=icon,
                                       label_pos=(0, 5))
        button.set_action_callback(menu_item.get_action_callback())
        return button
    
    def connect(self):
        win = self._skin.get_window()
        self._button_list.active_changed.connect(self.active_changed)
        key_pressed = self._button_list.key_pressed
        widget.Widget.sig_key_press_event.connect(key_pressed)
        widget.Widget.sig_key_press_event.neutralize((key_pressed,))
        
    def disconnect(self):
        win = self._skin.get_window()
        key_pressed = self._button_list.key_pressed
        widget.Widget.sig_key_press_event.disconnect(key_pressed)
        widget.Widget.sig_key_press_event.restore()
        self._button_list.active_changed.disconnect(self.active_changed)

    def _save(self):
        self._config_plugin.hide_panel()

    def _show_plugins(self):
        current_item = self._button_list.current_item
        menu_item = self._menu_mapping.get(current_item)
        if menu_item:
            self.flip_forward(menu_item)

    def _show_plugin(self):
        current_item = self._button_list.current_item
        menu_item = self._menu_mapping.get(current_item)
        if menu_item:
            self.flip_forward(menu_item)

    def _toggle_plugin(self):
        current_item = self._button_list.current_item
        menu_item = self._menu_mapping.get(current_item)
        if menu_item:
            plugin_item = menu_item.get_parent()
            plugin_name = plugin_item.get_short_name()
            plugin_manager = common.get_plugin_manager()
                
            plugin = plugin_manager.get_plugin_with_name(plugin_name)
            if not plugin.enabled:
                plugin_item.enable()
                plugin.enable()
                msg = _("Plugin %(plugin_name)s will be enabled next time Elisa starts")
            else:
                plugin_item.disable()
                plugin.disable()
                msg = _("Plugin %(plugin_name)s will be disabled next time Elisa starts")
                
            plugin.enabled = not plugin.enabled
            self._update_toggle_plugin(current_item, menu_item, plugin.enabled,
                                       plugin.name)
            #self.flip_backward()
            self._config_plugin.touch()
            self._skin.notify(msg % {'plugin_name': plugin.name})

    def _update_toggle_plugin(self, button, menu_item, enabled, plugin_name):
        if enabled:
            toggle_text = _('Disable plugin')
            help = _('Disable the %(plugin_name)s plugin')
        else:
            toggle_text = _('Enable plugin')
            help = _('Enable the %(plugin_name)s plugin')
        menu_item.set_short_name(toggle_text)
        menu_item.set_help_string(help % {'plugin_name':plugin_name})
        self.display_help(menu_item.get_help_string())        
        if button:
            button.text = toggle_text
            
    def flip_forward(self, menu_item):
        self.flip(True, menu_item)

    def flip_backward(self):
        current_item = self._button_list.current_item
        menu_item = self._menu_mapping.get(current_item)
        parent = menu_item.get_parent()
        if parent:
            p = parent.get_parent()
            if p:
                parent = p
        if not parent:
            parent = self._root_menu
            
        self.flip(False, parent)

    def flip(self, forward, menu_item):
        """
        Rotate the left side of the panel on its ry axis
        """
        if forward:
            start = 0
            end = 180            
        else:
            start = 180
            end = 0
            
        cb = lambda ctr: self.build(menu_item._items)
        start_fade = 1
        end_fade = -255
        
        timeline = [ keyframe.KeyFrame(start, 0),
                     keyframe.KeyFrame(end, 1) ]
        timeline_fade = [ keyframe.KeyFrame(start_fade, 0),
                          keyframe.KeyFrame(end_fade, 1)]
        
        prop = 'ry'
        duration = 2000

        # don't fade arrows neither background
        buttons = self._button_list.children[1:-1][:]
        buttons.remove(self._button_list.bg)
        mod_fade = modifier.Modifier(buttons, 'opacity_offset',
                                     timeline_fade)

        # rotate the whole button list
        mod = modifier.Modifier([self._button_list,],prop, timeline)
        
        self._ctrl = controller.Controller(begin=0, duration=duration,
                                           resolution=10,
                                           modifiers=[mod_fade, mod],
                                           end_behavior=controller.END,
                                           interpolation=controller.SINUSOIDAL,
                                           end_callback=cb)
        self._ctrl.start()

    def build(self, items):
        current_items = self._button_list

        # skip prev/next list items
        for item in current_items[1:-1]:
            self._button_list.remove(item)

        # back button
        back = menu.MenuItem(short_name=_("Back"))
        back.set_target_path("back")
        back.set_help_string(_("Save and go back to previous menu"))
        back.set_action_callback(self.flip_backward)

        parent = items[0].get_parent()        
        if parent != self._root_menu:
            if back.get_target_path() not in [m.get_target_path()
                                              for m in parent]:
                parent.insert_item(0, back)
                self.add_button(back, index=0)
            leaf = True
        else:
            items = self._root_menu._items
            leaf = False
            
        for item in items:
            self.add_button(item, leaf=leaf)

        self._button_list.select(1)
