# -*- coding: utf-8 -*-
# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php

# Copyright (C) 2006,2007, Elisa team <contact@fluendo.com>

from elisa.core.base_plugins import UserInputPlugin
from elisa.core import common
from elisa.core.interfaces import IUserInputPlugin
from elisa.core.signals.application import application_started
from pgm import render

import os
import copy
import pkg_resources

try:
    import pylirc
except ImportError:
    pylirc = None

class LIRCPlugin(UserInputPlugin):

    __implements__ = IUserInputPlugin
    __platforms__ = ['win32', 'posix']
        
    config_doc = {'lirc_rc': 'filename of the LIRC config map to use'}
    default_config = {'lirc_rc': 'streamzap.lirc',
                      }
    name = "lirc"

    provides = ['input',]

    singleton = True

    def __init__(self):
        UserInputPlugin.__init__(self, "remote_control")
        self.sock = None
        self.do_unload = False
        
        rc_file = self.get_lirc_config()
        if pylirc:
            self.logger.info("Using LIRC config file: %s" % rc_file)
            try:
                self.sock = pylirc.init('elisa', rc_file)
            except Exception, error:
                self.load_error = error
                self.do_unload = True
            else:
                pylirc.blocking(False)
        else:
            self.load_error = "pylirc package not found"
            self.do_unload = True
            
        application_started.connect(self._application_started)
        
    def post_init(self):
        if self.do_unload:
            self.logger.info(self.load_error)
            self.unload()

    def get_lirc_config(self):
        filename = self.get_config().get('lirc_rc','')
        appli = common.get_application()
        config_dir = appli.get_config().get_config_dir()

        lirc_config = os.path.join(config_dir, filename)
        if not os.path.exists(lirc_config):
            lirc_config = pkg_resources.resource_filename(self.__module__,
                                                          'data/%s' % filename)
        return lirc_config

    def _application_started(self):
        self.key_actions = {}
        pgmrender = render.get_renderer()
        appli = common.get_application()
        
        player = appli.get_player()
        skin = appli.get_skin()
        
        all_actions = copy.copy(appli.key_actions)
        all_actions.update(player.key_actions)
        if skin:
            all_actions.update(skin.key_actions)

        for pgm_key, value in all_actions.iteritems():
            if type(value) in (list, tuple):
                value = value[0]                
            self.key_actions[value] = pgm_key
            
    def get_input_events(self):
        input_events = []
        pgmrender = render.get_renderer()

        if self.sock:
            values = pylirc.nextcode(1)
            if values:
                values = values[0]
                nb_repeated = values['repeat']
                keycode = values['config'][:-4]
                pgm_key = self.key_actions.get(keycode)
                if pgm_key and (nb_repeated == 0 or nb_repeated >= 5):
                    press = self.create_pgm_render_event(pgmrender.KEY_PRESS)
                    press.keyval = pgm_key
                    release = self.create_pgm_render_event(pgmrender.KEY_RELEASE)
                    release.keyval = pgm_key
                    input_events = [press, release]
                else:
                    self.logger.info("Unsupported LIRC keycode: %s" % keycode)
        return input_events
