# -*- coding: utf-8 -*-
# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php

# Copyright (C) 2006,2007, Elisa team <contact@fluendo.com>

from elisa.core.interfaces import IMediaPlayer
from elisa.core.base_plugins import PlayerPlugin
from elisa.core.menu import MenuItem
from elisa.core import common, media_uri
from elisa.core.utils import image
from elisa.core.plugins import pictures

import elisa.core.utils.misc
import sys

class MusicTreePlugin(PlayerPlugin):
    """
    music plugin_tree
    """

    __implements__ = IMediaPlayer

    name = "music"
    menu_name = _("Music")
    menu_help_string = _("Listen to music")

    config_doc = {'locations': 'the media locations',
                  'cover_file': 'filename to display as album cover '\
                                'if available in an album directory'
                  }

    default_config = {'locations': ['file://./sample_data/music/*',
                                    'meta://artist:list'],
                      'cover_file': 'front.jpg'}

    provides = ["audio", "application"]

    preferred_formats = ['x-ms-wma','mpeg', 'ogg']
    item_icon_path = 'music-file.png'
    folder_icon_path = 'folder-music.png'

    def __init__(self):
        if sys.platform == "win32": #Add My Music to the default config
            from win32com.shell import shell
            clsid_mymusic = 13
            try:
                p = shell.SHGetSpecialFolderPath(0, clsid_mymusic)
                print "My Music directory is:", p
                if p:
                    p = media_uri.Uri(p)
                    self.default_config['locations'].append(p.get_location())
            except:
                print "Could not retrieve My Music directory"

        PlayerPlugin.__init__(self)        
        self.head_uri = []

        # create the playlist menu
        self._playlist_menu = MenuItem(short_name=_("Playlists"))
        icon_path = 'music.png'
        self._playlist_menu.set_icon_path(icon_path)
        self._playlist_menu.set_items_count(1)
        self._playlist_menu.set_loadmore_callback(self._get_queued_items)

    def _get_queued_items(self, uri, start_item, nb_items):
        items = []
        player = common.get_application().get_player()
        if player and player.get_queue():
            queue = player.get_queue()
            current, short_name = queue.get_current_item()
            while current:
                data_plugin = self.get_data_plugin(self.head_uri)
                if data_plugin:
                    create_menu = data_plugin.create_menu_for_uri
                    item = create_menu(current, self,
                                       short_name=short_name)
                    if item:
                        items.append(item)
                current, short_name = queue.next_item(self.head_uri, current)

        return items

    def play_all_item(self, menu_item):
        PlayerPlugin.play_all_item(self, menu_item)
        if self._playlist_menu not in self._items:
            self._items.extend([self._playlist_menu,])
        self.head_uri = menu_item.get_target_path()

    def play_item(self, menu_item):
        if not self.lock():
            return
        
        player = self.get_player()

        music_file = str(media_uri.Uri(location=menu_item.get_target_path()))
        player = self.get_player()
        current_playable = player.get_current_item()
        dfr = None

        def launch(r):
            self._check_slideshow()
            self.logger.info('Playing music %s' % music_file)
            player.play(start=True)
            player.un_mute()
            PlayerPlugin.play_item(self, menu_item)
            self.unlock()
            
        if current_playable and current_playable.get_uri() == music_file:
            if player.get_state() == player.PAUSED:
                player.play()
            else:
                dfr = player.set_uri(music_file)
        else:
            dfr = player.set_uri(music_file)

        if dfr:
            dfr.addCallback(launch)
        else:
            self.unlock()
            
    item_action = play_item

    def _check_slideshow(self):
        task_manager = common.get_application().get_task_manager()
        slideshows = task_manager.get_tasks_with_type(pictures.Slideshow)
        if slideshows:
            slideshows[0].abort_slideshow()
        
 
    def get_folder_thumbnail(self, uri, widget_item, size):
        cover_file = self.get_config().get('cover_file')
        cover_uri = media_uri.Uri("%s/%s" % (str(uri), cover_file))

        def img_thumbnailer(uri, size):
            thumbnail = image.Image(uri)
            thumbnail.resize(size)
            return thumbnail

        th_manager = common.get_application().get_thumbnail_manager()
        try:
            thumbnail = th_manager.get_thumbnail(cover_uri, size, img_thumbnailer)
            widget_item.set_background_image(thumbnail)
        except Exception, e:
            self.logger.debug("Thumbnail generation failed for %s: %s:" % (uri, str(e)))


    def add_folder_wizard(self, menu_item):
        pass


    def item_filter(self, uri):

        uri = media_uri.Uri(location=uri)
        data_plugin = self.get_data_plugin(uri)
        is_item = False
        if data_plugin:
            if uri.scheme == 'file':
                is_item = data_plugin.get_file_type(uri) == 'audio'
            else:
                is_item = not data_plugin.is_dir(str(uri))
            
        return is_item
