# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from elisa.core import common
from elisa.core.signals.menu import *
from pgm.timing import controller, modifier, keyframe
from pgm.layout import group
from elisa.core.widgets import widget_level
from elisa.core.plugins.skins.default import cached_widget_level

class WidgetMenu(group.Group):
    
    def __init__(self, current_skin, arrow_picture_path = None, width=800):
        group.Group.__init__(self)
        self._skin = current_skin
        self.width = width
        self._mm = current_skin.get_menu_manager()
        self._window = common.get_application().get_window()
        self._level_rank = 0
        self._level_offset = 50
        self._arrow_picture_path = self._skin.get_picture_path(arrow_picture_path)
        level_loading.connect(self._level_loading)
        level_loaded.connect(self._level_loaded)
        level_added.connect(self._level_added)
        
    def height__set(self, height):
        group.Group.height__set(self, height)

        # FIXME: why is this necessary?
        if not self.y:
            offset = 0
        else:
            offset = 40
        self.y = self._skin.get_window().canvas_size[1] - (height / 2.) - offset
        
    def unlock_animation(self):
        self._skin.logger.debug("Unlocking animation")
        self._mm.unlock()
                
    def get_current_widget_level(self):
        current_menu_level = self._mm.get_current_menu_item().get_parent()
        return self._mm.get_level_widget_for_menu_item( current_menu_level )
    
    def show_menu(self):
        """
        """
        self._skin.logger.info("Showing menu")
        self.visible = True
        self._mm.unlock()
        
    def hide_menu(self):
        """
        """
        self._skin.logger.info("Hiding menu")
        self.visible = False
        self._mm.lock()

    def toggle_menu(self):
        try:
            in_anim = self.get_current_widget_level()._anim_in_progress != None
        except AttributeError:
            in_anim = False
        return not in_anim
        
    def rotate_image(self, uri, angle):
        current_menu_level = self._mm.get_current_menu_item().get_parent()
        parent_menu_level = current_menu_level.get_parent()
        level_widget = self._mm.get_level_widget_for_menu_item(parent_menu_level)
        level_widget.rotate_image(uri, angle)
        
    def move_left(self):
        if self._mm.is_locked():
            self.get_current_widget_level().move_left()
            label = self._mm.get_current_menu_item().get_help_string()
            self._skin.get_status_bar().set_label(label)
            self._skin.hide_dock()
            return True
        return False
        
    def move_right(self):
        if self._mm.is_locked():
            self.get_current_widget_level().move_right()
            label = self._mm.get_current_menu_item().get_help_string()
            self._skin.get_status_bar().set_label(label)
            self._skin.hide_dock()
            return True
        return False
                   
    def move_up(self):
        if self._mm.is_locked():
            current_menu_item = self._mm.get_current_menu_item()        
            menu_level = current_menu_item
            widget_level = self._create_widget_level(menu_level)

            if widget_level:
                w_item = widget_level.get_widget_item(widget_level._active_item_index)
                if w_item:
                    w_item.set_current(False)
                    self.remove(widget_level)
                self._anim_level(menu_level, 'up')
                if self._level_rank:
                    self._level_rank -= 1
            label = self._mm.get_current_menu_item().get_help_string()
            self._skin.get_status_bar().set_label(label)
            
    def move_down(self):
        if self._mm.is_locked():
            current_menu_item = self._mm.get_current_menu_item()
            current_level_item = current_menu_item.get_parent()
            widget_item = self._mm.get_widget_item_for_menu_item(current_menu_item)
            if current_menu_item.has_items():
                widget_item.set_current(False)
                self._level_rank += 1
                self._anim_level(current_menu_item, 'down')
            else:
                self.unlock_animation()
                
        
    def _level_added(self, menu_item):
        if menu_item == self._mm.get_root_item():
            root_widget = self._create_widget_level(menu_item, 5)
            root_widget.set_status(widget_level.NORMAL)
        else:
            wlevel = self._create_widget_level(menu_item, 7, 5)
            wlevel.set_status(widget_level.NORMAL)
    
    def _get_translate_modifier(self, widget_list, start_x, end_x,
                                operation, property='x'):
        timeline_pos = [ keyframe.KeyFrame(start_x, 0.0),
                         keyframe.KeyFrame(end_x, 1.0) ]
        return modifier.Modifier(widget_list, property,
                                            timeline_pos, 
                                            operation=operation)
        
                
    def _anim_level(self, futur_menu_level, anim_type):               
        if anim_type == 'down':
            y_translate_value = -self._level_offset
            opacity_trans_value = -100
            z_translate_value = -500
            callback = self._anim_end_down            
        else:
            y_translate_value = self._level_offset
            opacity_trans_value = 100
            z_translate_value = 500
            callback = self._anim_end

            # Show label and arrow on current new level
            w_level = self.get_current_widget_level()
            w_item = w_level.get_widget_item(w_level._active_item_index)
            w_item.show_label()
            w_item._label.opacity = 255
            w_item.show_arrow()

        parent_mlevel_list = self._get_parent_level_list(futur_menu_level)
        parent_wlevel_list = []
        mdf_list = []
        for menu_level in parent_mlevel_list:
            wlevel = self._mm.get_level_widget_for_menu_item(menu_level)
            parent_wlevel_list.append(wlevel)
        mdf = self._get_translate_modifier (parent_wlevel_list, 0,
                                            y_translate_value,
                                            modifier.RELATIVE, 'y')
        mdf_list.append(mdf)
        mdf = self._get_translate_modifier (parent_wlevel_list, 0,
                                            z_translate_value,
                                            modifier.RELATIVE, 'tz')
        mdf_list.append(mdf)
        mdf = self._get_translate_modifier (parent_wlevel_list, 0,
                                            opacity_trans_value,
                                            modifier.RELATIVE, 'opacity_offset')
        mdf_list.append(mdf)
        ctrl = controller.Controller(begin=0, duration=250,
                                     resolution=15,
                                     end_behavior=controller.END,
                                     modifiers=mdf_list,
                                     end_callback=callback)
        ctrl.start() 
    
    def _anim_end(self, ctrl):
        menu_level = self._mm.get_current_menu_item().get_parent()
        cwidget_level = self._mm.get_level_widget_for_menu_item(menu_level)
        if cwidget_level and menu_level.has_items():
            cwidget_item = cwidget_level.get_widget_item(cwidget_level._active_item_index)
            if cwidget_item:
                cwidget_item.visible = True

        self.unlock_animation()

    def _anim_end_down(self, ctrl):
        menu_level = self._mm.get_current_menu_item()
        #check if item_widget yest existing in cache
        parent_level = self._mm.get_level_widget_for_menu_item(menu_level.get_parent())
        if parent_level:
            cwidget_item = parent_level.get_widget_item(parent_level._active_item_index)
            # Hide label and arrow on up level
            cwidget_item.hide_arrow()
            cwidget_item.hide_label()
                     
        dfr = self._mm.go_down_level(lock=False, requested_menu_items=8)
        
    def _get_parent_level_list(self, menu_item):
        parent_list = []
        root = self._mm.get_root_item()
        parent = menu_item.get_parent()
        while parent:
            parent_list.append(parent)
            if parent == root:
                break
            parent = parent.get_parent()
        return parent_list

    def _level_loading(self, menu_item):
        self._skin.start_loading_animation(menu_item)
        
    def _level_loaded(self, menu_item, nb_items):
        self._skin.stop_loading_animation()

        level_widget = self._mm.get_level_widget_for_menu_item(menu_item)
        if level_widget and nb_items>0:
            
            if menu_item != self._mm.get_root_item():
                level_widget.position = (0, -20)

                level_widget.set_status(widget_level.NORMAL)
                self.add(level_widget)
                level_widget.first_draw()

            else:
                level_widget.position = (0, -60)
                level_widget.set_status(widget_level.NORMAL)
                self.add(level_widget)
                level_widget.first_draw()
    
                current_menu_item = self._mm.get_current_menu_item()
                level_widget._level_loaded = True
                label = current_menu_item.get_help_string()
                self._skin.get_status_bar().set_label(label)
                
        if self._mm.is_locked():
            self._mm.unlock()
                        
    def _create_widget_level(self, menu_level=None, visible_item_count=5,
                             zoomed_items=3):
        if not menu_level:
            menu_level = self._mm.get_current_menu_item()
        # check if item_widget exists in cache 
        cwidget_level = self._mm.get_level_widget_for_menu_item(menu_level)
        if not cwidget_level:
            zoom_factor = 2
            arrow_path = self._arrow_picture_path
            image_path = self._skin.get_picture_path(arrow_path)
            wlevel = cached_widget_level.WidgetLevel(self._skin, menu_level,
                                                     image_path,
                                                     visible_item_count=visible_item_count,
                                                     zoomed_items=zoomed_items,
                                                     zoom_factor=zoom_factor)
            self._mm.bind_menu_level_to_widget(menu_level, wlevel)
        else:
            wlevel = cwidget_level
                        
        return wlevel
