# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from elisa.core import menu, log, media_uri
from elisa.core.signals.data import data_updated
from elisa.core.signals.task import new_task, del_task

class TaskManager(menu.MenuItem, list):

    def __init__(self):
        menu.MenuItem.__init__(self, "Tasks")
        list.__init__(self)
        self.logger = log.Logger()
        #self.hide()
        self.set_icon_path("folder.png")
        self.set_target_path("tasks")
        self.set_loadmore_callback(self._get_menuitems)
        self.set_help_string("Tasks management")

    def _get_menuitems(self, uri, start_item, nb_items):
        items = [ task.get_menu_item() for task in self ]
        return items

    __iter__ = list.__iter__
    
    def add_task(self, task):
        menu_item = task.get_menu_item()
        if menu_item not in [t.get_menu_item() for t in self]:
            self.logger.info("Adding task %s" % menu_item.get_short_name())
            self.add_control_menu(task)
            self.append(task)
            self.show()
            new_task.emit(task)
        
    def del_task(self, task):
        self.logger.info("Removing task %s" % task.get_menu_item().get_short_name())
        index = self.index(task)
        menu_item = task.get_menu_item()
        parent = menu_item.get_parent()
        if parent:
            data_updated.emit(menu_item.get_short_name(),
                              'del_file', menu_item.get_target_path(),
                              '', parent.get_target_path(), index)
            del_task.emit(task)
        self.remove(task)
        if len(self) == 0:
            self.hide()
            
    def get_task_with_target_path(self, target_path):
        for task in self:
            uri = media_uri.Uri(task.get_menu_item().get_target_path())
            if str(uri) == target_path:
                return task
        return None

    def get_tasks_with_type(self, task_type):
        tasks = []
        for task in self:
            if isinstance(task, task_type):
                tasks.append(task)
        return tasks

    def add_control_menu(self, task):
        menu_item = task.get_menu_item()

        if task.pause:
            pause = menu.MenuItem("pause")
            pause.set_icon_path('pause.png')
            pause.set_action_callback(task.pause)
            menu_item.add_item(pause)

        if task.resume:
            resume = menu.MenuItem("resume")
            resume.set_icon_path('play.png')
            resume.set_action_callback(task.resume)
            menu_item.add_item(resume)

        if task.abort:
            abort = menu.MenuItem("abort")
            abort.set_icon_path('stop.png')
            abort.set_action_callback(task.abort)
            menu_item.add_item(abort)
