# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

import threading
import urllib
import sys
import application
from elisa.core import log
from elisa.core import __version__

ELISA_UPDATE_PATH = 'http://elisa.fluendo.com/updates/' # always add trailing slash

class UpdateChecker(threading.Thread):
    def __init__(self, app):
        threading.Thread.__init__(self)
        self._app = app
        self._dl_url = None
        self.logger = log.Logger()

    def stringlist2intlist(self, l):
        i = 0
        while i < len(l):
            l[i] = int(l[i])
            i = i + 1
        return l

    def download_update(self):
        
        def ok_callback():
            pass
        
        if not self._dl_url:
            return
        try:
            t = urllib.urlretrieve(self._dl_url)
            self.logger.info("Update downloaded to %s" % t[0])
        except Exception, err:
            self.logger.info("Error downloading update: %s" % err)
            skin = self._app.get_skin()
            msg = _("An error occured while downloading the update,"\
                    " please download it manually" \
                    " from http://www.fluendo.com/elisa or try again later") 
            if skin:
                skin.open_popup(msg, ok_callback)
            else:
                print msg
            return
        if (t):
            from subprocess import Popen, PIPE
            p = Popen(t[0], shell=True, stdout=PIPE, stderr=PIPE)
            if (p):
                #FIXME: temp hack to kill elisa under win32, cause it won't close properly...
                if sys.platform == "win32":
                    from win32process import TerminateProcess, GetCurrentProcess
                    TerminateProcess(GetCurrentProcess(), 0)
                    #self._app.close()                                    

    def parse_update_file(self, update_file):
        
        def cancel_callback():
            pass
        
        tab = update_file.readlines()
        do_update = False
        exe_file = None
        for i in tab:
            i = i.strip(" \r\n")
            if i.startswith("#"):
                continue
            val = i.split("=")
            if len(val) == 2:
                if val[0] == "ver":
                    ver = self.stringlist2intlist(val[1].split("."))
                    ver_elisa = self.stringlist2intlist(__version__.split("."))
                    if ver > ver_elisa:
                        do_update = True
                        self.logger.info("New update available")
                    else:
                        do_update = False
                        self.logger.info("No new update available")
                elif val[0] == "file":
                    exe_file = val[1]
                        
        if do_update:
            if sys.platform == "win32" and exe_file:
                self._dl_url = ELISA_UPDATE_PATH + exe_file
                self._app.get_skin().open_popup(_("A new version of Elisa is available, do you want to download and install it now ?"),
                                                self.download_update, cancel_callback)
                self.logger.info("Downloading update...")
            else:
                self._app.get_skin().open_popup(_("A new version of Elisa is available, please download it from http://www.fluendo.com/elisa"),
                                                cancel_callback)

    def run(self):
        
        self.logger.info("Downloading update info")
        update_url = ELISA_UPDATE_PATH + "update.txt"
        f = None
        try:
            f = urllib.urlopen(update_url)
        except:
            self.logger.info("Could not download update file, no Internet connection?")            
        if f:
            self.parse_update_file(f)
        else:
            self.logger.info("Could not download update file")
        

