# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

import os
import sys
import locale
import socket
import struct
import pkg_resources

from twisted.internet import reactor

# see http://developer.gnome.org/doc/API/2.0/glib/glib-running.html
if "G_FILENAME_ENCODING" in os.environ:
    FSCODING = os.environ["G_FILENAME_ENCODING"].split(",")[0]
    if FSCODING == "@locale":
        FSCODING = locale.getpreferredencoding()
elif "G_BROKEN_FILENAMES" in os.environ:
    FSCODING = locale.getpreferredencoding()
else:
    FSCODING = "utf-8"

def get_host_address_win():
    import socket
    s = socket.gethostname()
    return socket.gethostbyname(s)

def get_host_address_posix():
    route_file = '/proc/net/route'
    route = open(route_file)
    if (route):
        tmp = route.readline() #skip first line
        while (tmp != ''):
            tmp = route.readline()
            l = tmp.split('\t')
            if (len(l) > 2):
                if l[2] != '00000000': #default gateway...
                    route.close()
                    return get_ip_address(l[0])
                
        route.close()
    return '127.0.0.1'

def get_host_address():
    import sys
    if sys.platform == "win32":
        return get_host_address_win()
    else:
        return get_host_address_posix()

def get_posix_ip_address(ifname):
    import fcntl
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        0x8915,  # SIOCGIFADDR
        struct.pack('256s', ifname[:15])
    )[20:24])    

def get_win32_ipaddress(ifname):
    #FIXME: to implement
    return ""

def get_ip_address(ifname):
    if sys.platform == "win32":
        return get_win32_ip_address(ifname)
    else:
        return get_posix_ip_address(ifname)

def handle_failure(f):
    """
    FIXME: log the exception and don't exit
    """
    assert f, "handle_failure takes a Failure, not None"
    print f.getTraceback()
    f.trap(Exception)
    reactor.stop()


def file_extension_matches(filename, pattern_list):
    path, extension = os.path.splitext(filename)
    # strip ext separator
    extension = extension[1:]
    extension = extension.lower()
    return extension in pattern_list

def file_is_picture(filename):
    return file_extension_matches(filename,('jpg', 'png', 'jpeg', 'gif'))

def file_is_movie(filename):
    return file_extension_matches(filename,('avi','mov','mpg','mpeg','vob',
                                            'asf','wmv', 'mkv', 'flv'))

def file_is_music(filename):
    return file_extension_matches(filename,('mp3','ogg','flac','wav'))


def fs_decode(s):
    """
    >>> fs_decode('sdkjhsdfjk')
    u'sdkjhsdfjkif'
    """
    if type(s) == unicode:
        return s
    else:
        return decode(s, FSCODING)

def fs_encode(s):
    """
    >>> fs_encode(u'sdkjhsdfjkiï')
    'sdkjhsdfjki\xc3\xaf'
    >>> fs_encode('sdkjhsdfjkiï')
    'sdkjhsdfjki\xc3\xaf'
    """
    if type(s) == str:
        return s
    else:
        return encode(s, FSCODING)

def decode(s, charset='utf-8'):
    try:
        return s.decode(charset)
    except UnicodeError:
        return s.decode(charset, 'replace')

def encode(s, charset='utf-8'):
    try:
        return s.encode(charset)
    except UnicodeError:
        return s.encode(charset, 'replace')

def get_resource_path(module, path):
    """
    This function must be used instead of pkg_resources.resource_filename,
    in order to permit us to also load the resources when the Elisa
    is frozen into an executable
    """
    if hasattr(sys, 'frozen') and sys.frozen:
        return os.path.join(os.path.dirname(sys.executable), path)
    else:
        return pkg_resources.resource_filename(module, path)
