# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from pgm.form import rectangle, label, color, image
from pgm.layout import group
from pgm import widget

import os, sys
sys.path.insert(0, os.getcwd())

from elisa.core.widgets.button import Button
from elisa.core.widgets.list import HList
from elisa.core import menu
from elisa.core.signals.menu import active_changed

class DialogBox(group.Group):

    def __init__(self, size, lines, background, font_name='Sans',
                 font_size=23, button_spacing=10, buttons_size=(50, 50),
                 modal=False, on_close=None, vertical_spacing=50):
        group.Group.__init__(self, size=size)

        self.v_spacing = vertical_spacing
        self.on_close = on_close
        
        # background, either a color or an image
        self.bg = rectangle.Rectangle(size=size, position=(0,0))
        self.bg.ordering = widget.FAR
        if isinstance(background, color.Color):
            self.bg.paint = background
        else:
            self.bg.media = background
        self.add(self.bg)

        self._label = None
        self.font_name = font_name
        self.font_size = font_size
        self.lines = lines
        self.buttons_size = buttons_size
        
        # buttons horiz list
        buttons_height = buttons_size[1] + 50
        self.buttons = HList((size[0], buttons_height), button_spacing)
        self.add(self.buttons)
        self.buttons.position = (0, size[1]-buttons_height)
        self.key_pressed = self.buttons.key_pressed
        
        self.set_modal(modal)
        
##     def text__set(self, txt):
##         """
##         FIXME: if len(txt) > 40: split text in lines and put each line in a
##                label. Until pigment:#55 is fixed.
##         """
##         if self._label:
##             self.remove(self._label)
##         self._label = label.Label(size=self.size, text=txt,
##                                   font_name=self.font_name,
##                                   font_size=self.font_size)
        
##         self._label.size = self._label.text_size
##         self._label.width = self.width
##         self.add(self._label)
##         self._label.x_align = 1.
##         self._label.ordering = widget.NEAR
##         self._label.y = 50

    def lines__set(self, lines):
        self._labels = []
        for line in lines:
            lbl = self.add_label(line)
            #lbl.visible = True
            self.add(lbl)
            self._labels.append(lbl)

    def visible__set(self, visible):
        group.Group.visible__set(self, visible)
        for lbl in self._labels:
            lbl.visible = visible
        
    def add_label(self, line):
        lbl = label.Label(size=self.size, text=line,
                          font_name=self.font_name,
                          font_size=self.font_size)
        
        lbl.y = 50 + len(self._labels) * self.v_spacing
        lbl.x_align = 0.5
        lbl.y_align = 0.
        lbl.ordering = widget.NEAR
        lbl.tz = 1
        lbl.opacity = 255
        return lbl
        
    def add_button(self, menu_item, on_press=None):
        icon = image.Image(menu_item.get_icon_path())
        button = Button(menu_item.get_short_name(),
                        icon=icon)
        button.size = self.buttons_size
        button.set_action_callback(menu_item.get_action_callback(), on_press)
        self.buttons.add(button)

    def close(self):
        if self._modal:
            widget.Widget.sig_key_press_event.restore()
        widget.Widget.sig_key_press_event.disconnect(self.key_pressed)
        #active_changed.disconnect(self.buttons._active_changed)
        if self.on_close:
            self.on_close(self)
        else:
            self.parent.remove(self)
            
    def set_modal(self, modal_value):
        self._modal = modal_value
        if modal_value:
            widget.Widget.sig_key_press_event.neutralize((self.key_pressed,))

    def center(self, x=True, y=True):
        parent = self.parent
        if x:
            new_x = (parent.canvas_size[0] - self.width) / 2.
            self.x = new_x
        if y:
            new_y = (parent.canvas_size[1] - self.height) / 2.
            self.y = new_y
        
    def set_default(self, index):
        for button in self.buttons:
            button.opacity = 128
        current = self.buttons[index]
        current.opacity = 255
        self.buttons.set_selected(index)

if __name__ == '__main__':

    from pgm import render
    from pgm.layout import window, group
    from pgm.form import rectangle, image
    import sys
    from pgm.timing import controller, modifier, keyframe
    from pgm import widget

    # why??
    if 'twisted.internet.reactor' in sys.modules:
        del sys.modules['twisted.internet.reactor']
        
    try:
        from twisted.internet import glib2reactor
        glib2reactor.install()
    except AssertionError,e:
        # already installed...
        print e
        raise
        pass
    
    from twisted.internet import reactor, task
    from twisted.spread import pb
    import gobject
    
    pgmrender = render.init('opengl1')

    
    def key_pressed(event):
        if event.keyval == pgmrender.KEY_q:
            reactor.stop()
            
    def refresh():
        win.dispatch_events()
        win.dispatch_controllers()
        win.update()
    
    def exit_event_cb(event):    
        win.main_quit()
        reactor.stop()
                    
    title = 'widget_level test'
    win = window.Window(title=title,size=(800,600))



    bg_color = color.Color((6,70,208,255))
    fg_color = color.Color((7, 182, 147, 255))

    bg = bg_color
    bg = image.Image('elisa/core/plugins/skins/default/pictures/popup.png')
    
    ok_path = 'elisa/core/plugins/skins/default/pictures/256X256/ok.png'
    cancel_path = 'elisa/core/plugins/skins/default/pictures/256X256/cancel.png'

    def on_ok():
        print 'ok!'

    def on_cancel():
        print 'cancel :('
    
    b_size = (100, 100)
    box = DialogBox((571, 365), ("To be? or not to be",),
                    bg, button_spacing=30,
                    buttons_size=b_size, font_size=35, modal=True)

    ok_menu_item = menu.MenuItem(short_name="")
    ok_menu_item.set_icon_path(ok_path)
    ok_menu_item.set_action_callback(on_ok)
    cancel_menu_item = menu.MenuItem(short_name="")
    cancel_menu_item.set_icon_path(cancel_path)
    cancel_menu_item.set_action_callback(on_cancel)
    
    box.add_button(ok_menu_item, box.close)
    box.add_button(cancel_menu_item, box.close)
    box.set_default(0)
    
    win.sig_key_press_event.connect(box.key_pressed)
    
    
    _background_widget = rectangle.Rectangle((800,600))
    _background_widget.ordering = widget.FAR
    img = image.Image('elisa/core/plugins/skins/default/pictures/background.png')
    _background_widget.media = img
    _background_widget.tz = -1
    win.add(_background_widget)
    
    win.add(box)
    box.center()

    win.sig_exit_event.connect(exit_event_cb)
    win.sig_key_press_event.connect(key_pressed)
    gobject.timeout_add(15, win.update)

    win.main(reactor.loop)
    
    reactor.run()
