# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 2.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from pgm.form import rectangle, label, color, image
from pgm.layout import group
from pgm import widget

class ProgressBar(group.Group):

    def __init__(self, size, foreground_color, background_color, opacity):
        self.progress_height = size[1] - 4
        self.progress = rectangle.Rectangle(size=(0, self.progress_height),
                                            position=(2,2),
                                            paint=foreground_color)

##         img_in = image.Image("progress_bar_in.png")
##         self.progress.media = img_in
        
        #self.progress.opacity = opacity
        #self.progress.ordering = widget.NEAR
        
        self.bg = rectangle.Rectangle(size=size, position=(0,0),
                                      paint=background_color)
##         img_bg = image.Image("progress_bar.png")
##         self.bg.media = img_bg
        
        self.bg.ordering = widget.FAR
      
        group.Group.__init__(self, size=size)
        self.add(self.bg)

        
        self._fraction = 0
        self.full_width = size[0]
        
    def fraction__set(self, fraction):
        fraction = float(fraction)
        if fraction >= -0.01 and fraction <= 1.01:
            self._fraction = fraction
            new_width = self.full_width * fraction
            self.progress.size = (new_width, self.progress_height)
            if self.progress not in self:
                self.add(self.progress)
                
    def fraction__get(self):
        return self._fraction
    
    def size__set(self, size):
        if self.progress:
            self.progress.size__set(size)
    
    def size__get(self):
        if self.progress:
            return self.progress.size
        return None
    
    def width__set(self, width):
        if self.progress:
            self.progress.width = width

    def height__set(self, height):
        if self.progress:
            self.progress.height = height

if __name__ == '__main__':

    from pgm import render
    from pgm.layout import window, group
    from pgm.form import rectangle, image
    import sys
    from pgm.timing import controller, modifier, keyframe
    from pgm import widget
    
    try:
        from twisted.internet import glib2reactor
        glib2reactor.install()
    except AssertionError:
        # already installed...
        pass
    
    from twisted.internet import reactor, task
    from twisted.spread import pb
    import gobject
    
    pgmrender = render.init('opengl1')

    bg_color = color.Color((6,70,208,255))
    fg_color = color.Color((7, 182, 147, 255))

    bar = ProgressBar((400, 30), fg_color, bg_color, 100)

    def key_pressed(event):
        step = 0.05
        if event.keyval == pgmrender.KEY_RIGHT:
            bar.fraction += step
        elif event.keyval == pgmrender.KEY_LEFT:
            bar.fraction -= step
            
    
    def exit_event_cb(event):    
        win.quit_loop()
        reactor.stop()
                    
    title = 'widget_level test'
    win = window.Window(title=title)
    
    
    _background_widget = rectangle.Rectangle((800,600))
    _background_widget.ordering = widget.FAR
    img = image.Image('elisa/core/plugins/skins/default/pictures/background.png')
    _background_widget.media = img
    _background_widget.tz = -1
    win.add(_background_widget)
    
    win.add(bar)
    bar.position = (10, 10)

    win.sig_exit_event.connect(exit_event_cb)
    win.sig_key_press_event.connect(key_pressed)
    gobject.timeout_add(15, win.update)

    win.main(reactor.loop)
    
    reactor.run()
